/* Straight from the Solaris FAQ */
 
#ifndef lint 
static char sccsid[] = "@(#)stubs.c for runme bpowell-  Titan 4.0.10 03/24/01";
#endif /* not lint */ 

char* dlopen() { return 0;}
int  dlclose() { return 0;}
char* dlsym() { return 0;}
char* dgettext() { return "";}
char* dlerror() { return "dynamic linking not loaded";}

